@@ -53,6 +53,7 @@ extern gchar *gnupg_homedir;
 extern gboolean cms_hack;
 extern gboolean disable_ticker;
 extern gboolean debug_edit_fsm;
+extern gboolean verbose;
 
 /* Show the keyring editor dialog.  */
 void gpa_open_key_manager (GtkAction *action, void *data);
@@ -76,9 +77,12 @@ static const GtkActionEntry gpa_windows_menu_action_entries[] =
       { "WindowsFileManager", GPA_STOCK_FILEMAN, NULL, NULL,
 	N_("Open the file manager"), G_CALLBACK (gpa_open_filemanager) },
       { "WindowsClipboard", GPA_STOCK_CLIPBOARD, NULL, NULL,
-	N_("Open the clipboard"), G_CALLBACK (gpa_open_clipboard) },
+	N_("Open the clipboard"), G_CALLBACK (gpa_open_clipboard) }
+#ifdef ENABLE_CARD_MANAGER
+      ,
       { "WindowsCardManager", GPA_STOCK_CARDMAN, NULL, NULL,
 	N_("Open the card manager"), G_CALLBACK (gpa_open_cardmanager) }
+#endif /*ENABLE_CARD_MANAGER*/
   };
 
 
@@ -105,6 +109,8 @@ typedef void (*GPADefaultKeyChanged) (gpointer user_data);
 void gpa_run_server_continuation (assuan_context_t ctx, gpg_error_t err);
 void gpa_start_server (void);
 void gpa_stop_server (void);
+int  gpa_check_server (void);
+gpg_error_t gpa_send_to_server (const char *cmd);
 
 
 typedef struct gpa_filewatch_id_s *gpa_filewatch_id_t;
@@ -131,6 +137,7 @@ char *decode_c_string (const char *src);
 char *percent_escape (const char *string,
                       const char *delimiters, int space2plus);
 size_t percent_unescape (char *string, int plus2space);
+void decode_percent_string (char *str);
 
 
 /*-- Convenience macros. -- */
