@@ -45,6 +45,7 @@
 #include "gpafilesignop.h"
 #include "gpafiledecryptop.h"
 #include "gpafileverifyop.h"
+#include "gpafileimportop.h"
 
 
 #define set_error(e,t) assuan_set_error (ctx, gpg_error (e), (t))
@@ -78,6 +79,9 @@ struct conn_ctrl_s
   /* File descriptor set with the MESSAGE command.  */
   int message_fd;
 
+  /* Flag indicating the the output shall be binary.  */
+  int output_binary;
+
   /* Channels used with the gpgme callbacks.  */
   GIOChannel *input_channel;
   GIOChannel *output_channel;
@@ -104,7 +108,6 @@ struct conn_ctrl_s
 
   /* The list of all files to be processed.  */
   GList *files;
-  gboolean files_finished;
 };
 
 
@@ -346,7 +349,6 @@ release_files (conn_ctrl_t ctrl)
   g_list_foreach (ctrl->files, (GFunc) free_file_item, NULL);
   g_list_free (ctrl->files);
   ctrl->files = NULL;
-  ctrl->files_finished = FALSE;
 }
 
 
@@ -535,6 +537,8 @@ prepare_io_streams (assuan_context_t ctx,
       err = gpgme_data_new_from_cbs (r_output_data, &my_gpgme_data_cbs, ctrl);
       if (err)
         goto leave;
+      if (ctrl->output_binary)
+        gpgme_data_set_encoding (*r_output_data, GPGME_DATA_ENCODING_BINARY);
     }
   if (ctrl->message_channel)
     {
@@ -1156,7 +1160,36 @@ cmd_start_keymanager (assuan_context_t ctx, char *line)
   return assuan_process_done (ctx, 0);
 }
 
+static const char hlp_start_clipboard[] =
+  "START_CLIPBOARD\n"
+  "\n"
+  "Pop up the clipboard window.  The client expects that the\n"
+  "clipboard is brought into the foregound and that this command\n"
+  "immediatley returns.";
+static gpg_error_t
+cmd_start_clipboard (assuan_context_t ctx, char *line)
+{
+  gpa_open_clipboard (NULL, NULL);
 
+  return assuan_process_done (ctx, 0);
+}
+
+static const char hlp_start_filemanager[] =
+  "START_FILEMANAGER\n"
+  "\n"
+  "Pop up the file manager window.  The client expects that the file\n"
+  "manager is brought into the foregound and that this command\n"
+  "immediatley returns.";
+static gpg_error_t
+cmd_start_filemanager (assuan_context_t ctx, char *line)
+{
+  gpa_open_filemanager (NULL, NULL);
+
+  return assuan_process_done (ctx, 0);
+}
+
+
+#ifdef ENABLE_CARD_MANAGER
 static const char hlp_start_cardmanager[] =
   "START_CARDMANAGER\n"
   "\n"
@@ -1170,7 +1203,7 @@ cmd_start_cardmanager (assuan_context_t ctx, char *line)
 
   return assuan_process_done (ctx, 0);
 }
-
+#endif /*ENABLE_CARD_MANAGER*/
 
 static const char hlp_start_confdialog[] =
   "START_CONFDIALOG\n"
@@ -1196,6 +1229,7 @@ static const char hlp_getinfo[] =
   "Supported values for WHAT are:\n"
   "\n"
   "  version     - Return the version of the program.\n"
+  "  name        - Return the name of the program\n"
   "  pid         - Return the process id of the server.";
 static gpg_error_t
 cmd_getinfo (assuan_context_t ctx, char *line)
@@ -1214,6 +1248,11 @@ cmd_getinfo (assuan_context_t ctx, char *line)
       snprintf (numbuf, sizeof numbuf, "%lu", (unsigned long)getpid ());
       err = assuan_send_data (ctx, numbuf, strlen (numbuf));
     }
+  else if (!strcmp (line, "name"))
+    {
+      const char *s = PACKAGE_NAME;
+      err = assuan_send_data (ctx, s, strlen (s));
+    }
   else
     err = set_error (GPG_ERR_ASS_PARAMETER, "unknown value for WHAT");
 
@@ -1221,98 +1260,24 @@ cmd_getinfo (assuan_context_t ctx, char *line)
 }
 
 
-
-/* Convert two hexadecimal digits from STR to the value they
-   represent.  Returns -1 if one of the characters is not a
-   hexadecimal digit.  */
-static int
-hextobyte (const char *str)
-{
-  int val = 0;
-  int i;
-
-#define NROFHEXDIGITS 2
-  for (i = 0; i < NROFHEXDIGITS; i++)
-    {
-      if (*str >= '0' && *str <= '9')
-        val += *str - '0';
-      else if (*str >= 'A' && *str <= 'F')
-        val += 10 + *str - 'A';
-      else if (*str >= 'a' && *str <= 'f')
-        val += 10 + *str - 'a';
-      else
-        return -1;
-      if (i < NROFHEXDIGITS - 1)
-        val *= 16;
-      str++;
-    }
-  return val;
-}
-
-
-/* Decode the percent escaped string STR in place.  */
-static void
-decode_percent_string (char *str)
-{
-  char *src = str;
-  char *dest = str;
-
-  /* Convert the string.  */
-  while (*src)
-    {
-      if (*src != '%')
-        {
-          *(dest++) = *(src++);
-          continue;
-        }
-      else
-        {
-          int val = hextobyte (&src[1]);
-
-          if (val == -1)
-            {
-              /* Should not happen.  */
-              *(dest++) = *(src++);
-              if (*src)
-                *(dest++) = *(src++);
-              if (*src)
-                *(dest++) = *(src++);
-            }
-          else
-            {
-              if (!val)
-                {
-                  /* A binary zero is not representable in a C
-                     string.  */
-                  *(dest++) = '\\';
-                  *(dest++) = '0';
-                }
-              else
-                *((unsigned char *) dest++) = val;
-              src += 3;
-            }
-        }
-    }
-  *(dest++) = 0;
-}
-
-
-/* FILE <file> [--continued]
-
-   Set the files on which to operate.
- */
+static const char hlp_file[] =
+  "FILE [--clear] <file>\n"
+  "\n"
+  "Add FILE to the list of files on which to operate.\n"
+  "With --clear given, that list is first cleared.";
 static gpg_error_t
 cmd_file (assuan_context_t ctx, char *line)
 {
   gpg_error_t err = 0;
   conn_ctrl_t ctrl = assuan_get_pointer (ctx);
-  gboolean continued;
+  gboolean clear;
   gpa_file_item_t file_item;
   char *tail;
 
-  continued = has_option (line, "--continued");
+  clear = has_option (line, "--clear");
+  line = skip_options (line);
 
-  if (ctrl->files_finished)
+  if (clear)
     release_files (ctrl);
 
   tail = line;
@@ -1325,13 +1290,12 @@ cmd_file (assuan_context_t ctx, char *line)
   file_item->filename_in = g_strdup (line);
   ctrl->files = g_list_append (ctrl->files, file_item);
 
-  if (! continued)
-    ctrl->files_finished = TRUE;
-
   return assuan_process_done (ctx, err);
 }
 
 
+/* Encrypt or sign files.  If neither ENCR nor SIGN is set, import
+   files. */
 static gpg_error_t
 impl_encrypt_sign_files (assuan_context_t ctx, int encr, int sign)
 {
@@ -1344,26 +1308,23 @@ impl_encrypt_sign_files (assuan_context_t ctx, int encr, int sign)
       err = set_error (GPG_ERR_ASS_SYNTAX, "no files specified");
       return assuan_process_done (ctx, err);
     }
-  else if (! ctrl->files_finished)
-    {
-      err = set_error (GPG_ERR_ASS_SYNTAX, "more files expected");
-      return assuan_process_done (ctx, err);
-    }
 
   /* FIXME: Needs a root window.  Need to set "sign" default.  */
   if (encr && sign)
     op = (GpaFileOperation *)
       gpa_file_encrypt_sign_operation_new (NULL, ctrl->files, FALSE);
-  if (encr)
+  else if (encr)
     op = (GpaFileOperation *)
       gpa_file_encrypt_operation_new (NULL, ctrl->files, FALSE);
-  else
+  else if (sign)
     op = (GpaFileOperation *)
       gpa_file_sign_operation_new (NULL, ctrl->files, FALSE);
+  else
+    op = (GpaFileOperation *)
+      gpa_file_import_operation_new (NULL, ctrl->files);
 
   /* Ownership of CTRL->files was passed to callee.  */
   ctrl->files = NULL;
-  ctrl->files_finished = FALSE;
   g_signal_connect (G_OBJECT (op), "completed",
 		    G_CALLBACK (g_object_unref), NULL);
 
@@ -1452,11 +1413,6 @@ impl_decrypt_verify_files (assuan_context_t ctx, int decrypt, int verify)
       err = set_error (GPG_ERR_ASS_SYNTAX, "no files specified");
       return assuan_process_done (ctx, err);
     }
-  else if (! ctrl->files_finished)
-    {
-      err = set_error (GPG_ERR_ASS_SYNTAX, "more files expected");
-      return assuan_process_done (ctx, err);
-    }
 
   /* FIXME: Needs a root window.  Need to enable "verify".  */
   if (decrypt && verify)
@@ -1471,7 +1427,6 @@ impl_decrypt_verify_files (assuan_context_t ctx, int decrypt, int verify)
 
   /* Ownership of CTRL->files was passed to callee.  */
   ctrl->files = NULL;
-  ctrl->files_finished = FALSE;
   g_signal_connect (G_OBJECT (op), "completed",
 		    G_CALLBACK (g_object_unref), NULL);
 
@@ -1567,10 +1522,10 @@ cmd_import_files (assuan_context_t ctx, char *line)
       return assuan_process_done (ctx, err);
     }
 
-  err = set_error (GPG_ERR_NOT_IMPLEMENTED, "not implemented");
-  return assuan_process_done (ctx, err);
+  return impl_encrypt_sign_files (ctx, 0, 0);
 }
 
+
 
 /* CHECKSUM_CREATE_FILES --nohup  */
 static gpg_error_t
@@ -1619,6 +1574,18 @@ cmd_checksum_verify_files (assuan_context_t ctx, char *line)
   return assuan_process_done (ctx, err);
 }
 
+
+
+/* KILL_UISERVER  */
+static gpg_error_t
+cmd_kill_uiserver (assuan_context_t ctx, char *line)
+{
+  (void)line;
+  shutdown_pending = TRUE;
+  return assuan_process_done (ctx, 0);
+}
+
+
 
 static gpg_error_t
 reset_notify (assuan_context_t ctx, char *line)
@@ -1637,6 +1604,7 @@ reset_notify (assuan_context_t ctx, char *line)
   assuan_close_output_fd (ctx);
   ctrl->input_fd = -1;
   ctrl->output_fd = -1;
+  ctrl->output_binary = 0;
   if (ctrl->gpa_op)
     {
       g_object_unref (ctrl->gpa_op);
@@ -1648,6 +1616,22 @@ reset_notify (assuan_context_t ctx, char *line)
   return 0;
 }
 
+
+static gpg_error_t
+output_notify (assuan_context_t ctx, char *line)
+{
+  conn_ctrl_t ctrl = assuan_get_pointer (ctx);
+
+  if (strstr (line, "--binary"))
+    ctrl->output_binary = 1;
+  else
+    ctrl->output_binary = 0;
+  /* Note: We also allow --armor and --base64 but because we don't
+     check for errors we don't need to parse them.  */
+  return 0;
+}
+
+
 
 /* Tell libassuan about our commands.   */
 static int
@@ -1670,10 +1654,14 @@ register_commands (assuan_context_t ctx)
     { "DECRYPT", cmd_decrypt, hlp_decrypt },
     { "VERIFY", cmd_verify, hlp_verify },
     { "START_KEYMANAGER", cmd_start_keymanager, hlp_start_keymanager },
-    { "START_CONFDIALOG", cmd_start_confdialog, hlp_start_confdialog },
+    { "START_CLIPBOARD", cmd_start_clipboard, hlp_start_clipboard },
+    { "START_FILEMANAGER", cmd_start_filemanager, hlp_start_filemanager },
+#ifdef ENABLE_CARD_MANAGER
     { "START_CARDMANAGER", cmd_start_cardmanager, hlp_start_cardmanager },
+#endif /*ENABLE_CARD_MANAGER*/
+    { "START_CONFDIALOG", cmd_start_confdialog, hlp_start_confdialog },
     { "GETINFO", cmd_getinfo, hlp_getinfo },
-    { "FILE", cmd_file },
+    { "FILE", cmd_file, hlp_file },
     { "ENCRYPT_FILES", cmd_encrypt_files },
     { "SIGN_FILES", cmd_sign_files },
     { "ENCRYPT_SIGN_FILES", cmd_encrypt_sign_files },
@@ -1683,6 +1671,7 @@ register_commands (assuan_context_t ctx)
     { "IMPORT_FILES", cmd_import_files },
     { "CHECKSUM_CREATE_FILES", cmd_checksum_create_files },
     { "CHECKSUM_VERIFY_FILES", cmd_checksum_verify_files },
+    { "KILL_UISERVER", cmd_kill_uiserver },
     { NULL }
   };
   int i, rc;
@@ -1738,6 +1727,7 @@ connection_startup (assuan_fd_t fd)
   assuan_set_pointer (ctx, ctrl);
   assuan_set_log_stream (ctx, stderr);
   assuan_register_reset_notify (ctx, reset_notify);
+  assuan_register_output_notify (ctx, output_notify);
   ctrl->message_fd = -1;
 
   connection_counter++;
@@ -2048,3 +2038,102 @@ gpa_stop_server (void)
   if (!connection_counter)
     gtk_main_quit ();
 }
+
+
+/* Helper for gpa_check-server.  */
+static gpg_error_t
+check_name_cb (void *opaque, const void *buffer, size_t length)
+{
+  int *result = opaque;
+  const char *name = PACKAGE_NAME;
+
+  if (length == strlen (name) && !strcmp (name, buffer))
+    *result = 1;
+
+  return 0;
+}
+
+
+/* Check whether an UI server is already running:
+   0  = no
+   1  = yes
+   2  = yes - same program
+ */
+int
+gpa_check_server (void)
+{
+  gpg_error_t err;
+  assuan_context_t ctx;
+  int name_check = 0;
+  int result;
+
+  err = assuan_new (&ctx);
+  if (!err)
+    err = assuan_socket_connect (ctx,
+                                 gpgme_get_dirinfo ("uiserver-socket"), 0, 0);
+  if (err)
+    {
+      if (verbose || gpg_err_code (err) != GPG_ERR_ASS_CONNECT_FAILED)
+        g_message ("error connecting an UI server: %s - %s",
+                   gpg_strerror (err), "assuming not running");
+      result = 0;
+      goto leave;
+    }
+
+  err = assuan_transact (ctx, "GETINFO name",
+                         check_name_cb, &name_check, NULL, NULL, NULL, NULL);
+  if (err)
+    {
+      g_message ("requesting name of UI server failed: %s - %s",
+                 gpg_strerror (err), "assuming not running");
+      result = 1;
+      goto leave;
+    }
+
+  if (name_check)
+    {
+      if (verbose)
+        g_message ("an instance of this program is already running");
+      result = 2;
+    }
+  else
+    {
+      g_message ("an different UI server is already running");
+      result = 1;
+    }
+
+ leave:
+  assuan_release (ctx);
+  return result;
+}
+
+
+/* Send a command to the server.  */
+gpg_error_t
+gpa_send_to_server (const char *cmd)
+{
+  gpg_error_t err;
+  assuan_context_t ctx;
+
+  err = assuan_new (&ctx);
+  if (!err)
+    err = assuan_socket_connect (ctx,
+                                 gpgme_get_dirinfo ("uiserver-socket"), 0, 0);
+  if (err)
+    {
+      g_message ("error connecting the UI server: %s", gpg_strerror (err));
+      goto leave;
+    }
+
+  err = assuan_transact (ctx, cmd, NULL, NULL, NULL, NULL, NULL, NULL);
+  if (err)
+    {
+      g_message ("error sending '%s' to the UI server: %s",
+                 cmd, gpg_strerror (err));
+      goto leave;
+    }
+
+ leave:
+  assuan_release (ctx);
+  return err;
+}
